Package: SingleCellExperiment
Version: 1.18.1
Date: 2022-08-29
Title: S4 Classes for Single Cell Data
Authors@R: c(
        person("Aaron", "Lun", role=c("aut", "cph"), email="infinite.monkeys.with.keyboards@gmail.com"),
        person("Davide","Risso", role=c("aut","cre", "cph"), email="risso.davide@gmail.com"),
        person("Keegan", "Korthauer", role="ctb"),
        person("Kevin", "Rue-Albrecht", role="ctb"))
Depends: SummarizedExperiment
Imports: methods, utils, stats, S4Vectors, BiocGenerics, GenomicRanges,
        DelayedArray
Suggests: testthat, BiocStyle, knitr, rmarkdown, Matrix, scRNAseq (>=
        2.9.1), Rtsne
biocViews: ImmunoOncology, DataRepresentation, DataImport,
        Infrastructure, SingleCell
Description: Defines a S4 class for storing data from single-cell
        experiments. This includes specialized methods to store and
        retrieve spike-in information, dimensionality reduction
        coordinates and size factors for each cell, along with the
        usual metadata for genes and libraries.
License: GPL-3
VignetteBuilder: knitr
RoxygenNote: 7.2.1
git_url: https://git.bioconductor.org/packages/SingleCellExperiment
git_branch: RELEASE_3_15
git_last_commit: db7768a
git_last_commit_date: 2022-09-30
Date/Publication: 2022-10-02
NeedsCompilation: no
Packaged: 2022-10-02 22:08:29 UTC; biocbuild
Author: Aaron Lun [aut, cph],
  Davide Risso [aut, cre, cph],
  Keegan Korthauer [ctb],
  Kevin Rue-Albrecht [ctb]
Maintainer: Davide Risso <risso.davide@gmail.com>
